<?php

namespace Drupal\pach\Entity;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManager as CoreEntityTypeManager;

/**
 * Overrides core EntityTypeManager to allow pluggable access control handlers.
 *
 * @see \Drupal\Core\Entity\EntityTypeManager
 */
class EntityTypeManager extends CoreEntityTypeManager {

  /**
   * {@inheritdoc}
   */
  public function getHandler($entity_type, $handler_type) {
    /* @var $handler_orig \Drupal\Core\Entity\EntityTypeInterface */
    $handler_orig = parent::getHandler($entity_type, $handler_type);
    if ('access' !== $handler_type) {
      // Do not alter other handler types.
      return $handler_orig;
    }
    if ($handler_orig instanceof PluggableAccessControlHandlerInterface) {
      // Handler is already altered.
      return $handler_orig;
    }

    // Replace original access control handler.
    /* @var $definition \Drupal\Core\Entity\EntityTypeInterface */
    $definition = $this->getDefinition($entity_type);
    $_class = $definition->getHandlerClass($handler_type);
    if (!$_class) {
      throw new InvalidPluginDefinitionException($entity_type, sprintf('The "%s" entity type did not specify a %s handler.', $entity_type, $handler_type));
    }
    $definition->setHandlerClass('_access', $_class);
    $class = 'Drupal\pach\Entity\EntityAccessControlHandler';
    $this->handlers[$handler_type][$entity_type] = $this->createHandlerInstance($class, $definition);
    return $this->handlers[$handler_type][$entity_type];
  }

  /**
   * {@inheritdoc}
   */
  public function createHandlerInstance($class, EntityTypeInterface $definition = NULL) {
    if (is_subclass_of($class, 'Drupal\Core\Entity\EntityHandlerInterface')) {
      $handler = $class::createInstance($this->container, $definition);
    }
    elseif (is_subclass_of($class, 'Drupal\pach\Entity\PluggableAccessControlHandlerInterface')) {
      $handler = $class::createInstance($this->container, $definition, $this);
    }
    else {
      $handler = new $class($definition);
    }
    if (method_exists($handler, 'setModuleHandler')) {
      $handler->setModuleHandler($this->moduleHandler);
    }
    if (method_exists($handler, 'setStringTranslation')) {
      $handler->setStringTranslation($this->stringTranslation);
    }

    return $handler;
  }

}
